//Record video from Onvif cameraersion 20230202
//LNT <lnt@ly-le.info>

const childProcess = require('child_process')
    , fs = require('fs')
//, digest = require('./digest')

const maxDelay = 8000
const minDelay = 3000
const imgType = '.jpg' // low cpu
const vidType = '.mkv'

class ffRecorder {
  opts = null
  wSt = null
  tId = null
  cnt = 0

  constructor (options) {
    this.opts = options
    if (options.prefix) this.opts.prefix += '_'
    if (!options.outFolder) this.opts.outFolder = '/cctv/' + this.opts.prefix
    if (!Number.isInteger(options.split)) this.opts.split = 0
    if (!options.logLevel) this.opts.logLevel = 'quiet'
    this.clip = [
      '-hide_banner',
      '-y',
      '-rtsp_transport', 'tcp',
      '-i', this.opts.streamUri
    ]

    if (this.opts.split == 2)
      this.clip = this.clip.concat('-vf', 'scale=640:-1', '-vf', 'fps=3/60', 'imgUrl')
    if (this.opts.split > 0) //split video
      this.clip = this.clip.concat('-f', 'segment', '-segment_time', 60)

    if (this.opts.audio)
      this.clip = this.clip.concat('-c:a', 'aac', '-b:a', '128k')
    else
      this.clip.push('-an')

    this.clip = this.clip.concat('-c:v', 'copy', '-preset', 'superfast', '')
  }

  mRecord (fn) {
    if (this.opts.split > 0) fn = fn + '_%03d'
    if (this.opts.split == 2) this.clip[10] = fn + imgType
    this.clip[this.clip.length -1] = fn + vidType
    return childProcess.spawn('ffmpeg', this.clip, {detached: false, stdio: 'ignore'})
  }

  fName() {
    let d = new Date()
    let a = d.toLocaleDateString().split('/')
    if (a[0].length == 1) a[0] = '0' + a[0]
    if (a[1].length == 1) a[1] = '0' + a[1]
    let d1 = a[2] + a[0] + a[1]
    let fn = this.opts.outFolder + '/' + d1
    if (!fs.existsSync(fn)) {
        fs.mkdirSync(fn, { recursive: true })
        if (this.opts.owner) childProcess.exec('chown ' + this.opts.owner + ' ' + fn)
    }
    let d2 = d.toLocaleTimeString('vi-VN').replace(/:/g,'')
    return fn + '/' + this.opts.prefix + d2
  }

  startRecording() {
    if (!this.wSt) {
      let fn = this.fName()
//    this.snapshot(fn + '_00' + imgType) //name of smapshot
      this.wSt = this.mRecord(fn)           //name of thumbnail and clip
      if (this.opts.owner) childProcess.exec( 'chown ' + this.opts.owner + ' ' + fn + '*' )
      if (this.opts.logLevel != 'quiet')
        console.log(' ', new Date().toLocaleTimeString('vi-VN'), '>> Start recording...')
      this.cnt = 0
    }
    if (this.tId) {
      clearTimeout(this.tId)
      this.tId = null
    }
    this.cnt++
  }
  // gọi từ event
  stopRecording() {
    if (!this.tId && this.wSt) {
      let delay = (this.cnt <= 0) ? minDelay : maxDelay
      this.tId = setTimeout(this.endRecording.bind(this), delay)
    }
  }

  endRecording() {
    if (this.tId) {
      clearTimeout(this.tId)
      this.tId = null
    }
    if (this.wSt) {
      this.wSt.kill()
      this.wSt = null
      if (this.opts.logLevel != 'quiet')
        console.log(' ', new Date().toLocaleTimeString('vi-VN'), '>> Stop recording!')
    }
    this.cnt = 0
  }
//  snapshot (fname) {
//    childProcess.spawn('wget', ['-q', '-b', '-O', fname, this.opts.imgUrl], {detached: false, stdio: 'ignore'})
//  childProcess.spawn('curl', ['--digest', '--output', fname, fUrl], {detached: false, stdio: 'ignore'})
//  new digest(fullUrl).request((res) => { res.pipe(fs.createWriteStream(fname)) })
//  }
}
module.exports = ffRecorder
